unit PrgSetup;
interface

uses Windows, IniFiles;

Type TWindowSetup=class
  private
    FSpezial : Boolean;
    FName : String;
    FMainWnd, FOutputWnd, FCPUWnd, FRAMWnd, FExtRAMWnd, FCalcWnd, FSeriellWnd, FMotorWnd : TRect;
    FOutput, FCPU, FRAM, FExtRAM, FCalc, FSeriell, FMotor : Boolean;
    Procedure SetName(S : String);
    Procedure SetStdData(Nr : Integer = 1);
  public
    Constructor Create(Vorgabe : Integer = 0); overload;
    Constructor Create(const Ini : TIniFile; const Section : String); overload;
    Procedure SaveData(const Ini : TIniFile);
    Procedure ReadRect(const Ini : TIniFile; const Name : String; Var R : TRect);
    Procedure SaveRect(const Ini : TIniFile; const Name : String; const R : TRect);
  published
    property Spezial : Boolean read FSpezial;
    property Name : String read FName write SetName;
    property MainWnd : TRect read FMainWnd write FMainWnd;
    property OutputWnd : TRect read FOutputWnd write FOutputWnd;
    property CPUWnd : TRect read FCPUWnd write FCPUWnd;
    property RAMWnd : TRect read FRAMWnd write FRAMWnd;
    property ExtRAMWnd : TRect read FExtRAMWnd write FExtRAMWnd;
    property CalcWnd : TRect read FCalcWnd write FCalcWnd;
    property SeriellWnd : TRect read FSeriellWnd write FSeriellWnd;
    property MotorWnd : TRect read FMotorWnd write FMotorWnd;
    property OutPut : Boolean read FOutput write FOutput;
    property CPU : Boolean read FCPU write FCPU;
    property RAM : Boolean read FRAM write FRAM;
    property ExtRAM : Boolean read FExtRAM write FExtRAM;
    property Calc : Boolean read FCalc write FCalc;
    property Seriell : Boolean read FSeriell write FSeriell;
    property Motor : Boolean read FMotor write FMotor;
end;

Type TSetupData=class
  private
    FIniFile : String;
    FWindowSetup : Array of TWindowSetup;
    FFontSize : Integer;
    FDebugSet : String;
    FStrictTable, FMemoryProtection, FAutoInterruptHandling : Boolean;
    FSingleMemory1,FSingleMemory2 : Boolean;
    FHighlightSyntax : Boolean;
    FSeriellStep : Integer;
    Procedure SetInteger(Index,Value : Integer);
    Procedure SaveIniData(const Ini : TIniFile);
    Procedure LoadIniData(const Ini : TIniFile);
    Function GetWindowSetupCount : Integer;
    Function GetWindowSetup(I : Integer) : TWindowSetup;
  public
    Constructor Create(AIniFile : String);
    Destructor Destroy; override;
    Function AddWindowSetup : Integer;
    Procedure DelWindowSetup(Nr : Integer);
    property WindowSetup[I : Integer] : TWindowSetup read GetWindowSetup;
  published
    property FontSize : Integer index 1 read FFontSize write SetInteger;
    property DebugSet : String read FDebugSet write FDebugSet;
    property WindowSetupCount : Integer read GetWindowSetupCount;
    property StrictTable : Boolean read FStrictTable write FStrictTable;
    property MemoryProtection : Boolean read FMemoryProtection write FMemoryProtection;
    property AutoInterruptHandling : Boolean read FAutoInterruptHandling write FAutoInterruptHandling;
    property SingleMemory1 : Boolean read FSingleMemory1 write FSingleMemory1;
    property SingleMemory2 : Boolean read FSingleMemory2 write FSingleMemory2;
    property HighlightSyntax : Boolean read FHighlightSyntax write FHighlightSyntax;
    property SeriellStep : Integer read FSeriellStep write FSeriellStep;
end;

implementation

uses Classes, Forms, Dialogs, PrgConst, Tools;

{ TSetupData }

constructor TSetupData.Create(AIniFile: String);
Var Ini : TIniFile;
begin
  inherited Create;
  SetLength(FWindowSetup,3);
  FWindowSetup[0]:=TWindowSetup.Create(1);
  FWindowSetup[1]:=TWindowSetup.Create(2);
  FWindowSetup[2]:=TWindowSetup.Create(3);
  FIniFile:=AIniFile;
  Ini:=TIniFile.Create(FIniFile);
  try LoadIniData(Ini); finally Ini.Free; end;
end;

destructor TSetupData.Destroy;
Var Ini : TIniFile;
begin
  try
    Ini:=TIniFile.Create(FIniFile);
    try SaveIniData(Ini); finally Ini.Free; end;
  except
    MessageDlg('Die Konfigurationseinstellungen konnten nicht gespeichert werden.',mtError,[mbOk],0);
  end;
  inherited Destroy;
end;

function TSetupData.GetWindowSetup(I: Integer): TWindowSetup;
begin
  result:=FWindowSetup[I];
end;

function TSetupData.GetWindowSetupCount: Integer;
begin
  result:=length(FWindowSetup);
end;

function TSetupData.AddWindowSetup: Integer;
begin
  SetLength(FWindowSetup,length(FWindowSetup)+1);
  result:=length(FWindowSetup)-1;
  FWindowSetup[result]:=TWindowSetup.Create;
end;

procedure TSetupData.DelWindowSetup(Nr: Integer);
Var X : Integer;
begin
  If (Nr<0) or (Nr>=length(FWindowSetup)) then exit;
  For X:=Nr+1 to length(FWindowSetup)-1 do FWindowSetup[X-1]:=FWindowSetup[X];
  SetLength(FWindowSetup,length(FWindowSetup)-1);
end;

procedure TSetupData.LoadIniData(const Ini: TIniFile);
Var St : TStrings;
    X : Integer;
begin
  FFontSize:=Ini.ReadInteger(StandardSection,'Fontsize',10);
  FDebugSet:=Ini.ReadString(StandardSection,'DebugWindowset','Debug');
  FStrictTable:=Ini.ReadBool(StandardSection,'AS31SymbolsOnly',False);
  FMemoryProtection:=Ini.ReadBool(StandardSection,'MemoryProtection',True);
  FAutoInterruptHandling:=Ini.ReadBool(StandardSection,'AutoInterruptHandling',True);
  FSingleMemory1:=Ini.ReadBool(StandardSection,'SingleMemory1',True);
  FSingleMemory2:=Ini.ReadBool(StandardSection,'SingleMemory2',True);
  HighlightSyntax:=Ini.ReadBool(StandardSection,'HighlightSyntax',True);
  FSeriellStep:=Ini.ReadInteger(StandardSection,'SeriellStep',1000);
  St:=TStringlist.Create; Ini.ReadSections(St);
  For X:=0 to St.Count-1 do If ExtUpperCase(St[X])<>ExtUpperCase(StandardSection) then begin
    SetLength(FWindowSetup,length(FWindowSetup)+1);
    FWindowSetup[length(FWindowSetup)-1]:=TWindowSetup.Create(Ini,St[X]);
  end;
  St.Free;
end;

procedure TSetupData.SaveIniData(const Ini: TIniFile);
Var St : TStrings;
    X : Integer;
begin
  Ini.WriteInteger(StandardSection,'Fontsize',FFontSize);
  Ini.WriteString(StandardSection,'DebugWindowset',FDebugSet);
  Ini.WriteBool(StandardSection,'AS31SymbolsOnly',FStrictTable);
  Ini.WriteBool(StandardSection,'MemoryProtection',FMemoryProtection);
  Ini.WriteBool(StandardSection,'AutoInterruptHandling',FAutoInterruptHandling);
  Ini.WriteBool(StandardSection,'SingleMemory1',FSingleMemory1);
  Ini.WriteBool(StandardSection,'SingleMemory2',FSingleMemory2);
  Ini.WriteBool(StandardSection,'HighlightSyntax',FHighlightSyntax);
  Ini.WriteInteger(StandardSection,'SeriellStep',FSeriellStep);
  St:=TStringlist.Create; Ini.ReadSections(St);
  For X:=0 to St.Count-1 do If ExtUpperCase(St[X])<>ExtUpperCase(StandardSection) then
    Ini.EraseSection(St[X]);
  For X:=0 to length(FWindowSetup)-1 do FWindowSetup[X].SaveData(Ini);
end;

procedure TSetupData.SetInteger(Index, Value: Integer);
begin
  Case Index of
    1 : FFontSize:=Value;
  end;
end;

{ TWindowSetup }

constructor TWindowSetup.Create(Vorgabe: Integer);
begin
  inherited Create;
  FSpezial:=(Vorgabe>0);
  Case Vorgabe of
    1 : FName:='Editor';
    2 : FName:='Ausgabe';
    3 : FName:='Debug';
  end;
  If Vorgabe>0 then SetStdData(Vorgabe) else SetStdData(1);
end;

constructor TWindowSetup.Create(const Ini: TIniFile; const Section: String);
begin
  inherited Create;
  SetStdData;
  FName:=Section;
  FOutput:=Ini.ReadBool(FName,'Output',False);
  FCPU:=Ini.ReadBool(Section,'ALU',False);
  FRAM:=Ini.ReadBool(FName,'RAM',False);
  FCalc:=Ini.ReadBool(FName,'Calc',False);
  FExtRAM:=Ini.ReadBool(FName,'ExtRAM',False);
  FSeriell:=Ini.ReadBool(FName,'Seriell',False);
  FMotor:=Ini.ReadBool(FName,'Motor',False);
  ReadRect(Ini,'Wnd',FMainWnd);
  ReadRect(Ini,'OutputWnd',FOutputWnd);
  ReadRect(Ini,'ALUWnd',FCPUWnd);
  ReadRect(Ini,'RAMWnd',FRAMWnd);
  ReadRect(Ini,'CalcWnd',FCalcWnd);
  ReadRect(Ini,'ExtRAMWnd',FExtRAMWnd);
  ReadRect(Ini,'SeriellWnd',FSeriellWnd);
  ReadRect(Ini,'MotorWnd',FMotorWnd);
end;

Procedure TWindowSetup.SetStdData(Nr : Integer);
Var R : TRect;
begin
  SystemParametersInfo(SPI_GETWORKAREA,0,@R,0);
  Case Nr of
    1 : begin
          FOutput:=False; FCPU:=False; FRAM:=True; FCalc:=True; FExtRam:=False; FSeriell:=False; FMotor:=False;
          FRAMWnd:=Rect(R.Right-200,R.Top+5,R.Right-5,R.Bottom-5);
          FCPUWnd:=Rect(FRAMWnd.Left-265-5,R.Top+5,FRAMWnd.Left-5,R.Top+140);
          FCalcWnd:=Rect(FRAMWnd.Left-345-5,R.Bottom-5-181,FRAMWnd.Left-5,R.Bottom-5);
          FMainWnd:=Rect(
                      FRAMWnd.Right-FRAMWnd.Left+5,
                      FCPUWnd.Bottom+5,
                      FRAMWnd.Left-5,
                      FCalcWnd.Top-5);
          FOutputWnd:=Rect(R.Left+5,R.Bottom div 3,R.Right div 3,R.Bottom-5);
          FExtRamWnd:=Rect(5,FCalcWnd.Top,FCalcWnd.Left-5,R.Bottom-5);
          FSeriellWnd:=Rect(5,5,250,FMainWnd.Top);
          FMotorWnd:=Rect(5,R.Bottom-550,530,R.Bottom-5);
        end;
    2 : begin
          FOutput:=True; FCPU:=False; FRAM:=False; FCalc:=False; FExtRam:=False; FSeriell:=False; FMotor:=False;
          FRAMWnd:=Rect(R.Right-200,R.Top+5,R.Right-5,R.Bottom-5);
          FCPUWnd:=Rect(FRAMWnd.Left-265-5,R.Top+5,FRAMWnd.Left-5,R.Top+140);
          FCalcWnd:=Rect(FRAMWnd.Left-345-5,R.Bottom-5-181,FRAMWnd.Left-5,R.Bottom-5);
          FMainWnd:=Rect(R.Left+5,R.Top+5,R.Right div 2 - 5, R.Bottom-5);
          FOutputWnd:=Rect(R.Right div 2 + 5,R.Top+5,R.Right-5,R.Bottom-5);
          FExtRamWnd:=Rect(5,FCalcWnd.Top,FCalcWnd.Left-5,R.Bottom-5);
          FSeriellWnd:=Rect(5,5,250,FMainWnd.Top);
          FMotorWnd:=Rect(5,R.Bottom-550,530,R.Bottom-5);
        end;
    3 : begin
          FOutput:=False; FCPU:=True; FRAM:=True; FCalc:=True; FExtRam:=True; FSeriell:=True; FMotor:=False;
          FRAMWnd:=Rect(R.Right-200,R.Top+5,R.Right-5,R.Bottom-5);
          FCPUWnd:=Rect(FRAMWnd.Left-265-5,R.Top+5,FRAMWnd.Left-5,R.Bottom-5);
          FCalcWnd:=Rect(FCPUWnd.Left-345-5,R.Bottom-5-181,FCPUWnd.Left-5,R.Bottom-5);
          FExtRamWnd:=Rect(5,FCalcWnd.Top-200,FCPUWnd.Left-5,FCalcWnd.Top);
          FMainWnd:=Rect(
                      5,
                      5,
                      FCPUWnd.Left-5,
                      FExtRAMWnd.Top-5);
          FOutputWnd:=Rect(R.Left+5,R.Bottom div 3,R.Right div 3,R.Bottom-5);
          FSeriellWnd:=Rect(5,FCalcWnd.Top,FCalcWnd.Left-5,R.Bottom-5);
          FMotorWnd:=Rect(5,R.Bottom-550,530,R.Bottom-5);
        end;
  end;
end;

procedure TWindowSetup.SaveData(const Ini: TIniFile);
begin
  If FSpezial then exit;
  Ini.WriteBool(FName,'Output',FOutput);
  Ini.WriteBool(FName,'ALU',FCPU);
  Ini.WriteBool(FName,'RAM',FRAM);
  Ini.WriteBool(FName,'Calc',FCalc);
  Ini.WriteBool(FName,'ExtRAM',FExtRAM);
  Ini.WriteBool(FName,'Seriell',FSeriell);
  Ini.WriteBool(FName,'Motor',FMotor);
  SaveRect(Ini,'Wnd',FMainWnd);
  SaveRect(Ini,'OutputWnd',FOutputWnd);
  SaveRect(Ini,'ALUWnd',FCPUWnd);
  SaveRect(Ini,'RAMWnd',FRAMWnd);
  SaveRect(Ini,'CalcWnd',FCalcWnd);
  SaveRect(Ini,'ExtRAMWnd',FExtRAMWnd);
  SaveRect(Ini,'MotorWnd',FMotorWnd);
end;

procedure TWindowSetup.ReadRect(const Ini: TIniFile; const Name: String; var R: TRect);
Var Re : TRect;
begin
  Re.Left:=Ini.ReadInteger(FName,Name+'X1',-1); If Re.Left=-1 then exit;
  Re.Top:=Ini.ReadInteger(FName,Name+'Y1',-1); If Re.Top=-1 then exit;
  Re.Right:=Ini.ReadInteger(FName,Name+'X2',-1); If Re.Right=-1 then exit;
  Re.Bottom:=Ini.ReadInteger(FName,Name+'Y2',-1); If Re.Bottom=-1 then exit;
  R:=Re;
end;


procedure TWindowSetup.SaveRect(const Ini: TIniFile; const Name: String; const R: TRect);
begin
  Ini.WriteInteger(FName,Name+'X1',R.Left);
  Ini.WriteInteger(FName,Name+'Y1',R.Top);
  Ini.WriteInteger(FName,Name+'X2',R.Right);
  Ini.WriteInteger(FName,Name+'Y2',R.Bottom);
end;

procedure TWindowSetup.SetName(S: String);
begin
  If not FSpezial then FName:=S;
end;

end.
