unit Rechner;
interface

uses Classes, Tools, ALU, RAM, Compiler;

Type TSystem8051=class
  private
    FData, FExt, FCode, FXRAM : TSpeicher;
    FSFRArea, FSFRArea2 : TSFRArea;
    FALU : TProzessor;
    FProgramm : TProgramm;
    FErrorCode : String;
    FErrorLine : Integer;
    FAnalogValue : Array[0..7] of Integer;
    Procedure SetAnalogValue(Nr : Integer; Value : Integer);
    Function GetAnalogValue(Nr : Integer) : Integer;
    Procedure ReadAnalog(Sender : TObject; Nr : Byte; var Value : Integer);
  public
    Constructor Create;
    Destructor Destroy; override;
    Function LoadCode(const Lines : TStrings; MemoryProtect, MiniOS : Boolean; Quite : Boolean = False) : Boolean;
    property Data : TSpeicher read FData;
    property Ext : TSpeicher read FExt;
    property Code : TSpeicher read FCode;
    property SFRArea : TSFRArea read FSFRArea;
    property SFRArea2 : TSFRArea read FSFRArea2;
    property XRAM : TSpeicher read FXRAM;
    property ALU : TProzessor read FALU;
    property Programm : TProgramm read FProgramm;
    property ErrorCode : String read FErrorCode;
    property ErrorLine : Integer read FErrorLine;
    property AnalogValue[Nr : Integer] : Integer read GetAnalogValue write SetAnalogValue;
end;

implementation

uses SysUtils, Dialogs;

{ TSystem8051 }

constructor TSystem8051.Create;
Var X : Integer;
begin
  inherited Create;
  FData:=TSpeicher.Create;
  FExt:=TSpeicher.Create;
  FCode:=TSpeicher.Create;
  FSFRArea:=TSFRArea.Create;
  FSFRArea2:=TSFRArea.Create(True);
  FXRAM:=TSpeicher.Create;
  FALU:=TProzessor.Create(FCode,FExt,FData,FSFRArea,FSFRArea2,FXRAM);
  FALU.OnReadAnalog:=ReadAnalog;
  FProgramm:=TProgramm.Create;
  FErrorCode:=''; FErrorLine:=0;
  For X:=0 to 7 do FAnalogValue[X]:=0;
end;

destructor TSystem8051.Destroy;
begin
  FALU.Free;
  FExt.Free;
  FData.Free;
  FCode.Free;
  FSFRArea.Free;
  FSFRArea2.Free;
  FXRAM.Free;
  FProgramm.Free;
  inherited Destroy;
end;

function TSystem8051.LoadCode(const Lines: TStrings; MemoryProtect, MiniOS : Boolean; Quite : Boolean): Boolean;
Var St : TStream;
    X : Integer;
begin
  FErrorCode:=''; FErrorLine:=0;
  try
    FProgramm.Compile(Lines,MemoryProtect);
  except
    on E : ECompilerError do begin
      FErrorCode:=E.Message; FErrorLine:=E.Line;
      result:=False; exit;
    end;
  end;
  result:=True;
  St:=FProgramm.GetAsmCode(MiniOS);
  FCode.LoadData(St,0);
  If FALU.SingleMemory1 then begin
    St.Position:=0;
    If St.Size>$7FFF then X:=$7FFF else X:=St.Size;
    FExt.LoadData(St,0,X);
  end;
  If FALU.SingleMemory2 and (St.Size>$7FFF) then begin
    St.Position:=$8000; X:=St.Size-$8000;
    FExt.LoadData(St,$8000,X);
  end;
  St.Free;
  If ((FProgramm.StartAddr<$100) or (FProgramm.StartAddr>=$7F00)) and not Quite then
    MessageDlg(
      'Die Startadresse ('+IntToHex(FProgramm.StartAddr,4)+') des Programmes '+
      'liegt in einem Bereich, der auf dem 8051 bereits fr Systemdaten '+
      'reserviert ist. Auf dem 8051 wrde der Versuch dieses Programm zu starten '+
      'zum Abstrz fhren.',mtWarning,[mbOK],0);


  FALU.Reg.Init(FProgramm.StartAddr);
  FALU.SeriellInBuffer:='';
  If FProgramm.ClearRAM then FData.Clear;
  If FProgramm.RAMFile<>'' then begin
    try
      St:=TFileStream.Create(FProgramm.RAMFile,fmOpenRead);
      FData.Clear;
      FData.LoadData(St,0);
      St.Free;
    except
      FErrorCode:='Die RAM-Datei "'+FProgramm.RAMFile+'" konnte nicht geladen werden.';
      FErrorLine:=0; result:=False;
    end;
  end;
  For X:=0 to FProgramm.RAMAddr.Count-1 do
    FData[Integer(FProgramm.RAMAddr[X])]:=Integer(FProgramm.RAMValue[X]);
  FData.ProtectList.Clear;
  For X:=0 to FProgramm.ProtectAddr.Count-1 do
    FData.ProtectList.Add(FProgramm.ProtectAddr[X]);

  FData.SetSize(256);
  FExt.SetSize(65536);
  FCode.SetSize(65536);
  FXRAM.SetSize(256);
  FSFRArea.SetSize(128);
end;

function TSystem8051.GetAnalogValue(Nr: Integer): Integer;
begin
  If (Nr>7) or (Nr<0) then result:=0 else result:=FAnalogValue[Nr];
end;

procedure TSystem8051.SetAnalogValue(Nr, Value: Integer);
begin
  If (Nr<=7) and (Nr>=0) then FAnalogValue[Nr]:=Value;
end;

procedure TSystem8051.ReadAnalog(Sender: TObject; Nr: Byte; var Value: Integer);
begin
  If Nr>7 then Value:=0 else Value:=FAnalogValue[Nr];
end;

end.
